<?php

/**
 * CacheDecorTest
 * 本测试依赖于ConfigureTest
 */
class CacheDecorTest extends CTestCase {
	
	
	public function setUp() {
		Yii::app()->cacheDecor->enable=true;
	}
	
	public function methodWithConfig($arg1,$arg2) {
		static $cacheDecorConfig=array('keys'=>'arg1');//只对arg1参数进行缓存，不考虑arg2是否变化
		return array($arg1,$arg2);
	}
	
	public function methodSameKey1($arg1,$arg2) {
		static $cacheDecorConfig=array('keys'=>'SameKey%(arg1)d');
		return $arg1+$arg2;
	}
	
	public function methodSameKey2($arg1,$arg2) {
		static $cacheDecorConfig=array('keys'=>'SameKey%(arg1)d');
		return $arg1+$arg2;
	}
	
	public function testGenKey() {
		$decor=Yii::app()->cacheDecor;
		
		$keys=array('CustomPrefix','arg1,arg2');
		$args=array('arg1'=>123,'arg2'=>456);
		$k=$decor->genKey($keys,$args);
		$this->assertEquals($k,$decor->genKey($keys,$args));
		
		$newArgs=$args;
		$newArgs['arg2']=1324123424;//change args,key not equals
		$this->assertNotEquals($k,$decor->genKey($keys,$newArgs));
		
		$keys[0]='NewCustomPrefix';//Change custom key prefix,not equals
		$this->assertNotEquals($k,$decor->genKey($keys,$args));
		
		
	}
	

	public function testRun() {
		
		
		$decor=Yii::app()->cacheDecor;
		

		$callback=array($this,'runCallback');
		$args=array(123);
		$this->assertEquals($decor->run($callback,$args),$args[0]);
		
		$randCallback=array($this,'runRandomCallback');
		$ret=$decor->run($randCallback);
		$this->assertEquals($ret,$decor->run($randCallback));//结果被缓存了
		
		$decor->enable=false;
		$this->assertEquals($args[0],$decor->run($callback,$args));//应该一致
		$this->assertNotEquals($ret,$decor->run($randCallback));//禁用时不应取缓存的结果
		$decor->enable=true;
		
		
		//获取静态变量中的配置测试
		
		$arg1=123;
		
		
		$this->assertEquals(
			$decor->run(array($this,'methodWithConfig'),array($arg1,1)),
			$decor->run(array($this,'methodWithConfig'),array($arg1,2))//arg2变化不影响缓存
		);
		 
		 
		 
		//测试pattern形式的共享Key
		$this->assertEquals(
			$decor->run(array($this,'methodSameKey1'),array(123,456)),
			$decor->run(array($this,'methodSameKey2'),array(123,456))
		);
		
		$ret=$decor->run(array($this,'methodSameKey1'),array(333,456));
		$this->assertEquals(
			$ret,
			$decor->run(array($this,'methodSameKey2'),array(333,789)) //不执行，取共享缓存
		);
		
		$this->assertEquals(
			Yii::app()->cache->get($decor->genKey('SameKey%(arg1)d',array('arg1'=>333))),
			$ret
		);
		
		$this->assertNotEquals(
			Yii::app()->cache->get($decor->genKey('SameKey%(arg1)d',array('arg1'=>456))),
			$ret
		);
		 
		
		
	}
	public function testCall() {
		$decor=Yii::app()->cacheDecor;
		$ret=$decor->call($this,'methodTest');
		$this->assertEquals($ret,$this->cache_deco_methodTest());
		$this->assertEquals($decor->call($this,'callWillFail'),CacheDecor::CALL_FAIL);//方法调用失败返回此标识
	}
	
	public function runCallback($arg1) {
		return $arg1;
	}
	
	public function runRandomCallback() {
		return sprintf("%s%s",microtime(true),rand(1,9999));
	}
	


	public function testGenSerialKey() {
		$decor=Yii::app()->cacheDecor;
		
		$keys=array('CustomKeyPrefix','arg1,arg2');
		$args=array('arg1'=>'Arg1','arg2'=>'Arg2');
		$ret=$decor->genSerialKey($keys,$args);
		$this->assertEquals(
			$ret,
			$decor->genSerialKey($keys,$args)
		);
		
		
		$keys=array('CustomKeyPrefix','arg1');
		$args2=array('arg1'=>'Arg1','arg2'=>'Arg2 Not Equals');
		$this->assertEquals(
			$decor->genSerialKey($keys,$args2),
			$decor->genSerialKey($keys,$args)
		);
	}
	public function testGenStrKey() {
		$decor=Yii::app()->cacheDecor;
		
		$keys='Autolabel_%(id)d_%(referer)s';
		$args=array('id'=>123,'referer'=>'http://www.sina.fk');
		$this->assertEquals($decor->genStrKey($keys,$args),$decor->genStrKey($keys,$args));
		
		$args2=$args;
		$args2['id']=456;
		$this->assertNotEquals($decor->genStrKey($keys,$args),$decor->genStrKey($keys,$args2));
		
	}
	
	public function testFormat() {
		$decor=Yii::app()->cacheDecor;
		$id=123;
		$referer='http://www.sina.gfw';
		$pattern='Autolabel_%(id)d_%(referer)s';
		$args=array(
			'id'=>$id,
			'referer'=>$referer
		);
		$this->assertEquals("Autolabel_{$id}_{$referer}",$decor->format($pattern,$args));
		
		$pattern='%(year)04d-%(month)02d';
		$this->assertEquals(
			date('Y-m'),
			$decor->format(
				$pattern,
				array('year'=>intval(date('Y')),'month'=>intval(date('m')))
			)
		);
	}
	

	
	public function cache_deco_methodTest() {//调用时使用methodTest方法名
		return 1234;
	}

}


